#include <hardware.h>

volatile int16_t adxl345_x_zero;
volatile int16_t adxl345_y_zero;
volatile int16_t adxl345_z_zero;

void  
adxl345_init (void)
{
  uint8_t  accel_data[2];

  accel_data[0] = DATA_FORMAT_REG;                /* Full resolution mode              */
  accel_data[1] = 0x0B;

  i2c_master_write (ACCEL_ADDR, accel_data, 2, 1);
  
  accel_data[0] = POWER_CTRL;                   /* take accelerometer out of standby mode    */
  accel_data[1] = 0x08;

  i2c_master_write (ACCEL_ADDR, accel_data, 2, 1);
  
  accel_data[0] = FIFO_CTRL;                  /* put FIFO into bypass mode           */
  accel_data[1] = 0x00;

  i2c_master_write (ACCEL_ADDR, accel_data, 2, 1);
}

int16_t  
adxl345_read_x (void)
{
  uint8_t  accel_data[2];
  uint16_t  x_axis_val;

  
  accel_data[0] = DATAX0;
  i2c_master_write (ACCEL_ADDR, accel_data, 1, 1);
  i2c_master_read (ACCEL_ADDR, accel_data, 2, 1);
  
  x_axis_val  = accel_data[1] << 8;
  x_axis_val   += accel_data[0];

  return ((int16_t) x_axis_val);
}

int16_t  
adxl345_read_y (void)
{
  uint8_t  accel_data[2];
  uint16_t  y_axis_val;

  
  accel_data[0] = DATAY0;
  i2c_master_write (ACCEL_ADDR, accel_data, 1, 1);
  i2c_master_read (ACCEL_ADDR, accel_data, 2, 1);
  
  y_axis_val  = accel_data[1] << 8;
  y_axis_val   += accel_data[0];

  return ((int16_t) y_axis_val);
}

int16_t  
adxl345_read_z (void)
{
  uint8_t  accel_data[2];
  uint16_t  z_axis_val;

  
  accel_data[0] = DATAZ0;
  i2c_master_write (ACCEL_ADDR, accel_data, 1, 1);
  i2c_master_read (ACCEL_ADDR, accel_data, 2, 1);
  
  z_axis_val  = accel_data[1] << 8;
  z_axis_val   += accel_data[0];

  return ((int16_t) z_axis_val);
}

void  
adxl345_calibrate (void)
{
  uint8_t  indx;


  adxl345_x_zero = 0;
  adxl345_y_zero = 0;
  adxl345_z_zero = 0;
  
  for (indx = 0; indx < 8; indx++) 
  {
    adxl345_x_zero += adxl345_read_x();
    adxl345_y_zero += adxl345_read_y();
    adxl345_z_zero += adxl345_read_z();
  }
  
  adxl345_x_zero = adxl345_x_zero >> 3;
  adxl345_y_zero = adxl345_y_zero >> 3;
  adxl345_z_zero = adxl345_z_zero >> 3;
}


