#include <hardware.h>

void  
i2c_init (void)
{
  uint32_t delay;


  MSTP (RIIC0)       =  0;     

  PORT1.ICR.BIT.B2     =  1;
  PORT1.ICR.BIT.B3     =  1;
  RIIC0.ICCR1.BIT.ICE  =  0;      
  RIIC0.ICCR1.BIT.IICRST =  1;      

  for (delay = 0; delay < 1000000; delay++);

  RIIC0.ICCR1.BIT.IICRST =  0;      
  RIIC0.ICSER.BIT.SAR0E  =  0;      
  RIIC0.ICMR1.BIT.CKS  =  3;      
  RIIC0.ICBRL.BYTE     =  0xfd;
  RIIC0.ICBRH.BYTE     =  0xf8;    
  RIIC0.ICMR3.BIT.ACKWP  =  1;      
  RIIC0.ICFER.BIT.TMOE   =  0;      
  RIIC0.ICFER.BIT.MALE   =  1;      
  RIIC0.ICCR1.BIT.ICE  =  1;     
}


void  
i2c_master_read (uint8_t addr, int8_t  *p_data, size_t len, int start)
{
  uint32_t  dummy;
  uint8_t  *recv_data;
  size_t   bytes_left;


  recv_data    = p_data;
  bytes_left = len;

  if (start)
  {
    while (RIIC0.ICCR2.BIT.BBSY == 1);
    RIIC0.ICCR2.BIT.ST = 1;
  } 
  else 
  {
    while (RIIC0.ICCR2.BIT.BBSY == 0);
    RIIC0.ICCR2.BIT.RS = 1;      
    while (RIIC0.ICCR2.BIT.RS == 1);
  }


  while (RIIC0.ICSR2.BIT.TDRE == 0);

  RIIC0.ICDRT    = (addr << 1 ) | 0x01;

  while (RIIC0.ICSR2.BIT.RDRF == 0);

  if (RIIC0.ICSR2.BIT.NACKF == 0)
  {
    if (len > 1) 
    {
      dummy = RIIC0.ICDRR;

      while (RIIC0.ICSR2.BIT.RDRF == 0);

      while (bytes_left > 2) 
      {
         bytes_left--;
         *recv_data = RIIC0.ICDRR;
         recv_data++;
         while (RIIC0.ICSR2.BIT.RDRF == 0);
      }

      RIIC0.ICMR3.BIT.WAIT  = 1;
      RIIC0.ICMR3.BIT.ACKBT = 1;
      
      *recv_data        = RIIC0.ICDRR;
      recv_data++;
    } 
    else 
    {
      RIIC0.ICMR3.BIT.WAIT  = 1;
      RIIC0.ICMR3.BIT.ACKBT = 1;
      
      dummy         = RIIC0.ICDRR;
    }
      
    while (RIIC0.ICSR2.BIT.RDRF == 0);

    RIIC0.ICSR2.BIT.STOP = 0;  
    RIIC0.ICCR2.BIT.SP   = 1;
     *recv_data         = RIIC0.ICDRR;
    recv_data++;
    RIIC0.ICMR3.BIT.WAIT = 0;

  } 
  else 
  {
    RIIC0.ICSR2.BIT.STOP = 0;        
    RIIC0.ICCR2.BIT.SP   = 1;
    dummy        = RIIC0.ICDRR;
  }

  while (RIIC0.ICSR2.BIT.STOP == 0);

  RIIC0.ICSR2.BIT.NACKF  = 0;

  RIIC0.ICSR2.BIT.STOP   = 0;

}


void  
i2c_master_write (uint8_t addr, int8_t  *p_data, size_t len, int stop)
{
  uint8_t  *send_data;
  size_t   bytes_left;


  send_data    = p_data;
  bytes_left = len;

  while (RIIC0.ICCR2.BIT.BBSY == 1);

  RIIC0.ICCR2.BIT.ST = 1;            
  
  while (RIIC0.ICSR2.BIT.TDRE == 0);

  RIIC0.ICDRT    = addr << 1;

  while (bytes_left > 0) 
  {
    if (RIIC0.ICSR2.BIT.NACKF == 0) 
    {
      if (RIIC0.ICSR2.BIT.TDRE == 1) 
      {  
        RIIC0.ICDRT = *p_data;
        p_data++;
        bytes_left--;
      }
    } 
    else 
      break;
  }

  if (bytes_left == 0) 
    while (RIIC0.ICSR2.BIT.TEND == 0);

  RIIC0.ICSR2.BIT.STOP  = 0;
  
  if (stop)
  {
    RIIC0.ICCR2.BIT.SP  = 1;

    while (RIIC0.ICSR2.BIT.STOP == 0);
  }

  RIIC0.ICSR2.BIT.NACKF = 0;
  RIIC0.ICSR2.BIT.STOP  = 0;
}

