#include "videogen.h"
#include "bmp.h"

void
delay (void)
{
  int i;
  
  for (i = 0; i < BLINK_CYCLES; ++i)
    __asm__ __volatile__ ("nop");
}

void
die_bad_format (void)
{
  int i;
  int leds[] = {6, 7, 10, 11, 14, 15};
  
  while (1)
  {
    for (i = 0; i < 6; ++i)
      led_on (leds[i]);

    delay ();
    
    for (i = 0; i < 6; ++i)
      led_off (leds[i]);
    
    delay ();
  }
}

void
fine (void)
{
  int i;
  int leds[] = {4, 5, 8, 9, 12, 13};
  
  for (i = 0; i < 6; ++i)
    led_on (leds[i]);
}

void
die_bad_bpp (void)
{
  int i;
  int leds[] = {6, 10, 14};
  
  while (1)
  {
    for (i = 0; i < 3; ++i)
      led_on (leds[i]);

    delay ();
    
    for (i = 0; i < 3; ++i)
      led_off (leds[i]);
    
    delay ();
  }
}

void
die_compressed (void)
{
  int i;
  int leds[] = {6, 10, 14};
  
  while (1)
  {
    for (i = 0; i < 3; ++i)
    {
      led_on (leds[i]);
      led_off (leds[i] + 1);
    }
    
    delay ();
    
    for (i = 0; i < 3; ++i)
    {
      led_off (leds[i]);
      led_on (leds[i] + 1);
    }
    
    delay ();
  }
}



void
bmp_to_buffer16 (const void *bitmap,
	         uint16_t *output,
		 int out_width, int out_height,
		 int xrepeat, int yrepeat,
		 uint16_t low, uint16_t high)
{
  const struct bmp_header *header = (struct bmp_header *) bitmap;
  const struct bmp_v5header *dib  = (struct bmp_v5header *) (bitmap + sizeof (struct bmp_header));
  
  const uint8_t *data;
  uint16_t value;
  
  int i, j, oi, oj, x, y;
  int width;
  
  if (header->magic != BMP_MAGIC)
    die_bad_format ();

  if (dib->bpp != 8)
    die_bad_bpp ();

  if (dib->compression)
    die_compressed ();

  width = ((dib->width >> 2) + !!(dib->width & 3)) << 2;

  data = bitmap  + header->offset;

  /* Blank output buffer */
  for (j = 0; j < out_height; ++j)
    for (i = 0; i < out_width; ++i)
      output[i + j * out_width] = low;
  
  for (j = 0; j < dib->height; ++j)
    for (i = 0; i < dib->width; ++i)
    {
      value = ((high - low) * data[i + (dib->height - j - 1) * width]) / 255 + low;

      for (oj = 0; oj < yrepeat; ++oj)
      {
	if ((y = j * yrepeat + oj) >= out_height)
	  goto loop_exit;

	for (oi = 0; oi < xrepeat; ++oi)
	{
	  if ((x = i * xrepeat + oi) >= out_width)
	    goto loop_exit;

	  output[x + y * out_width] = value;
	}
      }
    }

loop_exit:

  lcd_printf ("BMP: %dx%d, %d bpp",
	      dib->width, dib->height, dib->bpp);
  fine ();
}
