#include <videogen.h>
#include <pic.h>

void INT_Excep_TMR0_CMI0A (void) __attribute__ ((interrupt));

int state  = STATE_LINE;
int ticks  = 0;
int line   = ODD_FIELD_LINE_START;
int oddity = FIELD_ODD;
int y      = 0;
int hidy   = 0;
int current_tick_limit   = LINE_TICKS;
int current_line_stop    = ODD_FIELD_LINE_STOP;
int last_line_tick_limit = LINE_TICKS;

/* A ser calculados al arranque */
uint16_t eqpulse[EQPULSE_TICKS];

uint16_t vsync[VSYNC_TICKS];

uint16_t hsync[HSYNC_TICKS] =
{
  /* Pórtico anterior */
  BLANK_LEVEL, BLANK_LEVEL, BLANK_LEVEL,

  /* Sincronismo */
  SYNC_LEVEL, SYNC_LEVEL, SYNC_LEVEL,
  SYNC_LEVEL, SYNC_LEVEL, SYNC_LEVEL,
  SYNC_LEVEL,

  /* Pórtico posterior */
  BLANK_LEVEL, BLANK_LEVEL, BLANK_LEVEL,
  BLANK_LEVEL, BLANK_LEVEL, BLANK_LEVEL,
  BLANK_LEVEL, BLANK_LEVEL, BLANK_LEVEL
};

int pixind = 0;

uint16_t buffer[VERT_RESOLUTION * LINE_TICKS];

void
INT_Excep_TMR0_CMI0A (void)
{
  uint16_t value;

  switch (state)
  {
  case STATE_LINE:
    value = buffer[pixind++];
    
    /* Determinamos si conmutamos el estado */
    if (++ticks >= current_tick_limit)
    {
      ticks = 0;

      if (line == current_line_stop)
      {
	line  = 0;
	state = STATE_PREEQ;
      }
      else
      {
	y = ++hidy >> 2;
	
	++line;
	
	state = STATE_HSYNC;
      }
    }
    
    break;
    
  case STATE_HSYNC:
    value = hsync[ticks];

    if (++ticks >= HSYNC_TICKS)
    {
      ticks = 0;
      state = STATE_LINE;
      
      if (line == current_line_stop)
	current_tick_limit = last_line_tick_limit;
    }

    break;
    
  case STATE_PREEQ:
    value = eqpulse[ticks];
    
    if (++ticks >= EQPULSE_TICKS)
    {
      ticks = 0;
      
      if (++line >= EQPULSE_COUNT)
      {
	line  = 0;
	state = STATE_VSYNC;
      }
    }
    
    break;

  case STATE_VSYNC:
    value = vsync[ticks];
    
    if (++ticks >= VSYNC_TICKS)
    {
      ticks = 0;
      
      if (++line >= VSYNC_COUNT)
      {
	line  = 0;
	state = STATE_POSEQ;
      }
    }
    
    break;

  case STATE_POSEQ:
    value = eqpulse[ticks];
    
    if (++ticks >= EQPULSE_TICKS)
    {
      ticks = pixind = 0;
      
      if (++line >= EQPULSE_COUNT)
      {
	oddity = !oddity;
        hidy = y = 0;

	current_tick_limit = LINE_TICKS;
	
	if (oddity)
	{
	  state = STATE_HSYNC;
	  line  = ODD_FIELD_LINE_START;
	  current_line_stop = ODD_FIELD_LINE_STOP;
	  last_line_tick_limit = LINE_TICKS;
	}
	else
	{
	  state = STATE_LINE;
	  line  = EVEN_FIELD_LINE_START;
	  ticks = pixind = HALF_LINE_NOSYN_TICKS;
	  current_line_stop = EVEN_FIELD_LINE_STOP;
	  last_line_tick_limit = HALF_LINE_TICKS;
	}
      }
    }
    
    break;

  }

  
  DAC_SET (value);
}

void
pal_init (void)
{
  int i, j;

  /* Inicialización del pulso de igualación */
  for (i = 0; i < EQPULSE_TICKS; ++i)
  {
    if (i < EQPULSE_BLANK_TICKS)
      eqpulse[i] = BLANK_LEVEL;
    else if (i < (EQPULSE_BLANK_TICKS + EQPULSE_SYNC_TICKS))
      eqpulse[i] = SYNC_LEVEL;
    else
      eqpulse[i] = BLANK_LEVEL;
  }

  /* Inicialización del pulso de sincronismo */
  for (i = 0; i < VSYNC_TICKS; ++i)
  {
    if (i < VSYNC_PULSE_TICKS)
      vsync[i] = SYNC_LEVEL;
    else
      vsync[i] = BLANK_LEVEL;
  }

  /* Inicialización de la imagen en pantalla */

/*  for (j = 0; j < VERT_RESOLUTION; ++j)
    for (i = 0; i < LINE_TICKS; ++i)
    buffer[j * LINE_TICKS + i] = (i & 1) ^ !!(j & 8)? WHITE_LEVEL : BLACK_LEVEL; */

  bmp_to_buffer16 (bin2c_me_bmp_data,
		   buffer,
		   LINE_TICKS,
		   VERT_RESOLUTION,
		   1, 4,
		   BLACK_LEVEL, WHITE_LEVEL);
}
