/*
 * videogen.h: headers, prototypes and declarations for videogen
 * Creation date: Fri Sep  6 16:16:39 2013
 */

#ifndef _MAIN_INCLUDE_H
#define _MAIN_INCLUDE_H

#include <config.h> /* General compile-time configuration parameters */
#include <hardware.h> /* From renesas: Renesas RX62N cross-compile dependency */

/* Si estamos con el amplificador operacional, debemos
   rebajar los niveles de voltaje, o se provocará
   su saturación y no será capaz de mantener el
   voltaje. */

#define MODE_OPAMP

#ifdef MODE_OPAMP

#  define SYNC_LEVEL  0
#  define BLANK_LEVEL 90
#  define BLACK_LEVEL 90
#  define WHITE_LEVEL 300

#else

/* En el caso contrario, basta (y necesitamos)
   enviar la máxima tensión posible por el DAC */
#  define SYNC_LEVEL  0
#  define BLANK_LEVEL 308
#  define BLACK_LEVEL 308
#  define WHITE_LEVEL 1023

#endif

/* Duraciones en ticks de los intervalos más importantes de la
   señal PAL */
#define LINE_TICKS            77
#define HALF_LINE_TICKS       29 /* Contando el sincronismo */
#define HALF_LINE_NOSYN_TICKS 48 /* Sin contar el sincronismo */ 
#define HSYNC_TICKS           19 

/* Duraciones de los pulsos de igualación */
#define EQPULSE_BLANK_TICKS   5
#define EQPULSE_SYNC_TICKS    4
#define EQPULSE_SPACE_TICKS   39
#define EQPULSE_TICKS         (EQPULSE_BLANK_TICKS + EQPULSE_SYNC_TICKS + EQPULSE_SPACE_TICKS)

/* Duraciones de los pulsos de sincronismo vertical */
#define VSYNC_PULSE_TICKS     41
#define VSYNC_SPACE_TICKS     7
#define VSYNC_TICKS           (VSYNC_PULSE_TICKS + VSYNC_SPACE_TICKS)

#define EQPULSE_COUNT         5 /* Cinco pulsos de igualación */
#define VSYNC_COUNT           5 /* Y cinco de sincronismo */

#define ODD_FIELD_LINE_START  6
#define ODD_FIELD_LINE_STOP   310

#define FIELD_EVEN            0
#define FIELD_ODD             1

#define EVEN_FIELD_LINE_START 318
#define EVEN_FIELD_LINE_STOP  623

#define PAL_LINES             625
#define FIELD_LINES           (PAL_LINES >> 1)
#define VERT_RESOLUTION       FIELD_LINES

#define STATE_LINE  0 /* Línea activa */
#define STATE_HSYNC 1 /* Pórtico, sincronismo, etc */ 
#define STATE_PREEQ 2 /* Impulsos de pre-igualación */
#define STATE_VSYNC 3 /* Impulsos de sincronismo vertical */
#define STATE_POSEQ 4 /* Impulsos de post-igualación */

void pal_init (void);
void timer_init (void);
void bmp_to_buffer16 (const void *bitmap,
		      uint16_t *output,
		      int out_width, int out_height,
		      int xrepeat, int yrepeat,
		      uint16_t low, uint16_t high);

#endif /* _MAIN_INCLUDE_H */
